"""Use Typer to run gatorgrade to run the checks and generate the yml file."""

import sys
from pathlib import Path
from typing import Tuple

import typer
from rich.console import Console
from rich.emoji import Emoji

from gatorgrade.input.parse_config import parse_config
from gatorgrade.output.output import run_checks

# create an app for the Typer-based CLI

# define the emoji that will be prepended to the help message;
# note that this uses a Rich emoji so that it is as platform-
# independent as possible, across three major operating systems
gatorgrade_emoji = Emoji.replace(":crocodile:")

# create a Typer app that
# --> does not support completion
# --> has a specified help message with an emoji
app = typer.Typer(
    add_completion=False,
    help=f"{gatorgrade_emoji} Run the GatorGrader checks in the specified gatorgrade.yml file.",
)


# create a default console for printing with rich
console = Console()

# define constants used in this module
FILE = "gatorgrade.yml"
FAILURE = 1


@app.callback(invoke_without_command=True)
def gatorgrade(
    ctx: typer.Context,
    filename: Path = typer.Option(FILE, "--config", "-c", help="Name of the yml file."),
    report: Tuple[str, str, str] = typer.Option(
        (None, None, None),
        "--report",
        "-r",
        help="A tuple containing the following REQUIRED values: \
            1. The destination of the report (either file or env) \
            2. The format of the report (either json or md) \
            3. the name of the file or environment variable\
            4. use 'env md GITHUB_STEP_SUMMARY' to create GitHub job summary in GitHub Action",
    ),
    run_status_bar: bool = typer.Option(
        False,
        "--status-bar",
        help="Enable a progress bar for checks running/not running.",
    ),
    no_status_bar: bool = typer.Option(
        False, "--no-status-bar", help="Disable the progress bar entirely."
    ),
):
    """Run the GatorGrader checks in the specified gatorgrade.yml file."""
    # if ctx.subcommand is None then this means
    # that, by default, gatorgrade should run in checking mode
    if ctx.invoked_subcommand is None:
        # parse the provided configuration file
        checks = parse_config(filename)
        # there are valid checks and thus the
        # tool should run them with run_checks
        if len(checks) > 0:
            checks_status = run_checks(checks, report, run_status_bar, no_status_bar)
        # no checks were created and this means
        # that, most likely, the file was not
        # valid and thus the tool cannot run checks
        else:
            checks_status = False
            console.print()
            console.print(f"The file {filename} either does not exist or is not valid.")
            console.print("Exiting now!")
            console.print()
        # at least one of the checks did not pass or
        # the provided file was not valid and thus
        # the tool should return a non-zero exit
        # code to designate some type of failure
        if checks_status is not True:
            sys.exit(FAILURE)


if __name__ == "__main__":
    app()



"""Generates a list of commands to be run through gatorgrader."""

from collections import namedtuple
from pathlib import Path
from typing import Any
from typing import List

import yaml

from gatorgrade.input.set_up_shell import run_setup

# Represent data for a check from the configuration file.
# Every check will have data (`check`) and some may also have a `file_context`,
# which is a file path associated with the check to be used when running the check.
CheckData = namedtuple("CheckData", ["file_context", "check"])

# define the default encoding
DEFAULT_ENCODING = "utf8"


def parse_yaml_file(file_path: Path) -> List[Any]:
    """Parse a YAML file and return its contents as a list of dictionaries."""
    # confirm that the file exists before attempting to read from it
    if file_path.exists():
        # read the contents of the specified file using the default
        # encoding and then parse that file using the yaml package
        with open(file_path, encoding=DEFAULT_ENCODING) as file:
            # after parsing with the yaml module, return a list
            # of all of the contents specified in the file
            data = yaml.load_all(file, Loader=yaml.FullLoader)
            return list(data)
    # some aspect of the file does not exist
    # (i.e., wrong file or wrong directory)
    # and thus parsing with YAML is not possible;
    # return a blank list that calling function handles
    return []


def reformat_yaml_data(data):
    """Reformat the raw data from a YAML file into a list of tuples."""
    reformatted_data = []
    if len(data) == 2:
        setup_commands = data.pop(0)  # Removes the setup commands
        run_setup(setup_commands)
    add_checks_to_list(None, data[0], reformatted_data)
    return reformatted_data


def add_checks_to_list(path, data_list, reformatted_data):
    """Recursively loop through the data and add checks that are found to the reformatted list."""
    current_path = path  # Saves the current path to keep track of the location
    for ddict in data_list:
        for item in ddict:
            if isinstance(
                ddict[item], list
            ):  # Checks if the current dictionary has another list as its value
                if not path:
                    path = item
                else:
                    path = f"{path}/{item}"
                add_checks_to_list(
                    path, ddict[item], reformatted_data
                )  # Runs this same function on the list inside of a dictionary
                path = current_path
            else:  # Adds the current check to the reformatted data list
                reformatted_data.append(CheckData(file_context=path, check=ddict))
                break



"""Generate a YAML file with default messages and specific paths."""

import os
from typing import Dict
from typing import List

import typer
import yaml


def input_correct(initial_path_list: List[str], run_path: str) -> Dict:
    """Correct user-written paths."""
    # Recognize the paths users provide are the concise versions.
    # Unify the ending format to avoid different users' different input
    corrected_path = []
    # Run_path unify
    if run_path.endswith(os.path.sep) is False:
        run_path += os.path.sep
    for path in initial_path_list:
        # Combine the running path with the target path
        # To make sure the target path starts from the running directory
        path = run_path + path
        # Treat the last unit of the path as a concise name unit
        if path.endswith(os.path.sep) is False:
            path += os.path.sep
        corrected_path.append(path)
    # Convert list to dictionary for faster iteration
    return dict.fromkeys(corrected_path, "")


def create_targeted_paths_list(
    target_path_list: List[str], relative_run_path: str = "."
) -> List[str]:
    """Generate a list of targeted paths by walking the paths."""
    targeted_paths = []
    corrected_paths = input_correct(target_path_list, relative_run_path)
    # Go through the root repo, the sub dictionaries and files
    # The os.walk will only scan the paths
    # So the empty folders containing nothing won't be gone through
    for dirpath, _, filenames in os.walk(relative_run_path):
        # Split path string into multiple layers of directories
        path_dir_list = dirpath.split(os.path.sep)
        # Ignore folder starting with double underscore
        if any(path.startswith("__") for path in path_dir_list):
            continue
        # Ignore hidden folders and first layer. the root repo is always dot
        # Keep double dot. It means going back to the parent folder
        if any(
            path.startswith(".") and not path.startswith("..")
            for path in path_dir_list[1:]
        ):
            continue
        for filename in filenames:
            # Ignore special files
            if filename.startswith("__") or filename.startswith("."):
                continue
            # Combine the path with file name to get a complete path
            complete_actual_path = os.path.join(dirpath, filename) + os.path.sep
            for target in corrected_paths:
                if target in complete_actual_path:
                    polished_paths = complete_actual_path.replace(
                        f"{relative_run_path}{os.path.sep}", ""
                    )
                    targeted_paths.append(polished_paths)

    # If any of the user inputted file does not exist in any directory,
    # throw an exception indicating failure
    if not targeted_paths:
        typer.secho(
            "FAILURE: None of the user-provided file paths are"
            " found in the provided directory and the 'gatorgrade.yml' is NOT generated",
            fg=typer.colors.RED,
            err=True,
        )
        raise typer.Exit(1)

    # If some of the files are found and some are not found,
    # output a warning message saying which files were not found
    targeted_paths_string = " ".join(targeted_paths)
    for key in target_path_list:
        if key not in targeted_paths_string:
            typer.secho(
                f"WARNING \u26a0: '{key}' file path is not FOUND!"
                f"\nAll file paths except '{key}' are successfully"
                " generated in the 'gatorgrade.yml' file",
                fg=typer.colors.YELLOW,
            )
            return targeted_paths

    # If all the files exist in the root directory, print out a success message
    if targeted_paths:
        typer.secho(
            "SUCCESS \U0001f525: All the file paths were"
            " successfully generated in the 'gatorgrade.yml' file!",
            fg=typer.colors.GREEN,
        )

    return targeted_paths


def write_yaml_of_paths_list(
    path_names: List[str], search_root: str
):  # expected input: A path list
    """Write YAML file to create gatorgrade file and set default messages."""
    files_list = []
    # Create an empty list to store dictionaries
    for file_path in path_names:
        # Iterate through items in path_names
        if file_path.endswith(os.path.sep):
            file_path = file_path[0:-1]
        # Convert file separators to '/'
        file_path_fixed = file_path.replace(os.path.sep, "/")
        # Make file_path easier to read by removing unnecessary characters
        file_path_dict = {
            # Dictionary to store the file paths
            file_path_fixed: [
                # List which stores strings which will be in gatorgrade.yml file
                {
                    "description": f"Complete all TODOs in {file_path_fixed}",
                    "check": "MatchFileFragment",
                    "options": {"fragment": "TODO", "count": 0, "exact": True},
                }
            ]
        }
        # Append files_list with the values stored inside file_path_dict
        files_list.append(file_path_dict)

    with open(
        f"{search_root}{os.path.sep}gatorgrade.yml", "w", encoding="utf-8"
    ) as file:
        # Write a new YAML file named gatorgrade
        yaml.dump(files_list, file, sort_keys=False)
        # Dump strings stored in files_list into a new YAML file


def generate_config(target_path_list: List[str], search_root: str = "."):
    """Generate config by creating targeted paths in a list of strings, then create a YAML file."""
    targeted_paths = create_targeted_paths_list(target_path_list, search_root)
    write_yaml_of_paths_list(targeted_paths, search_root)



"""Set-up the shell commands."""

import subprocess

import typer


def run_setup(front_matter):
    """Run the shell set up commands and exit the program if a command fails.

    Args:
        front_matter: A dictionary whose 'setup' key contains the set up commands
        as a multi-line string.

    """
    # If setup exists in the front matter
    setup = front_matter.get("setup")
    if setup:
        typer.echo("Running set up commands...")
        for line in setup.splitlines():
            # Trims the white space
            command = line.strip()
            # Executes the command
            proc = subprocess.run(command, shell=True, check=False, timeout=300)
            # If the exit code tells it was unsuccessful and did not return 0
            if proc.returncode != 0:
                typer.secho(
                    f'The set up command "{command}" failed.\
                Exiting GatorGrade.',
                    err=True,
                    fg=typer.colors.RED,
                )
                # If a set up command failed, exit the execution
                # because environment was not set up correctly.
                raise typer.Exit(1)
        typer.echo("Finished!\n")



"""Run checks and display whether each has passed or failed."""

import datetime
import json
import os
import subprocess
from pathlib import Path
from typing import List
from typing import Tuple
from typing import Union

import gator
import rich
from rich.progress import BarColumn
from rich.progress import Progress
from rich.progress import TextColumn
from rich.panel import Panel

from gatorgrade.input.checks import GatorGraderCheck
from gatorgrade.input.checks import ShellCheck
from gatorgrade.output.check_result import CheckResult

# Disable rich's default highlight to stop number coloring
rich.reconfigure(highlight=False)


def _run_shell_check(check: ShellCheck) -> CheckResult:
    """Run a shell check.

    Args:
        check: The shell check to run.

    Returns:
        The result of running the shell check as a CheckResult.
    """
    result = subprocess.run(
        check.command,
        shell=True,
        check=False,
        timeout=300,
        stdout=subprocess.PIPE,
        # Redirect STDERR to STDOUT so STDOUT and STDERR can be captured
        # together as diagnostic
        stderr=subprocess.STDOUT,
    )
    passed = result.returncode == 0
    # Add spaces after each newline to indent all lines of diagnostic
    diagnostic = (
        "" if passed else result.stdout.decode().strip().replace("\n", "\n     ")
    )
    return CheckResult(
        passed=passed,
        description=check.description,
        json_info=check.json_info,
        diagnostic=diagnostic,
    )


def _run_gg_check(check: GatorGraderCheck) -> CheckResult:
    """Run a GatorGrader check.

    Args:
        check: The GatorGrader check to run.

    Returns:
        The result of running the GatorGrader check as a CheckResult.
    """
    try:
        result = gator.grader(check.gg_args)
        passed = result[1]
        description = result[0]
        diagnostic = result[2]
        # Fetch the path from gatorgrade arguments
        # the path pattern are 4 consistent string in the list
        # --dir `dir_name` --file `file_name`
        file_path = None
        for i in range(len(check.gg_args)):
            if check.gg_args[i] == "--directory":
                dir_name = check.gg_args[i + 1]
                file_name = check.gg_args[i + 3]
                file_path = dir_name + "/" + file_name
                break
    # If arguments are formatted incorrectly, catch the exception and
    # return it as the diagnostic message
    # Disable pylint to catch any type of exception thrown by GatorGrader
    except Exception as command_exception:  # pylint: disable=W0703
        passed = False
        description = f'Invalid GatorGrader check: "{" ".join(check.gg_args)}"'
        diagnostic = f'"{command_exception.__class__}" thrown by GatorGrader'
        file_path = None
    return CheckResult(
        passed=passed,
        description=description,
        json_info=check.json_info,
        diagnostic=diagnostic,
        path=file_path,
    )


def create_report_json(
    passed_count,
    checkResults: List[CheckResult],
    percent_passed,
) -> dict:
    """Take checks and put them into json format in a dictionary.

    Args:
        passed_count: the number of checks that passed
        check_information: the basic information about checks and their params
        checkResults: the list of check results that will be put in json
        percent_passed: the percentage of checks that passed
    """
    # create list to hold the key values for the dictionary that
    # will be converted into json
    overall_key_list = ["amount_correct", "percentage_score", "report_time", "checks"]
    checks_list = []
    overall_dict = {}
    report_generation_time = datetime.datetime.now()
    formatted_time = report_generation_time.strftime("%Y-%m-%d %H:%M:%S")
    # for each check:
    for i in range(len(checkResults)):
        # grab all of the information in it and add it to the checks list
        results_json = checkResults[i].json_info
        results_json["status"] = checkResults[i].passed
        if checkResults[i].path:
            results_json["path"] = checkResults[i].path
        if not checkResults[i].passed:
            results_json["diagnostic"] = checkResults[i].diagnostic
        checks_list.append(results_json)
    # create the dictionary for all of the check information
    overall_dict = dict(
        zip(
            overall_key_list,
            [passed_count, percent_passed, formatted_time, checks_list],
        )
    )
    return overall_dict


def create_markdown_report_file(json: dict) -> str:
    """Create a markdown file using the created json to use in github actions summary, among other places.

    Args:
        json: a dictionary containing the json that should be converted to markdown
    """
    markdown_contents = ""
    passing_checks = []
    failing_checks = []
    num_checks = len(json.get("checks"))  # type: ignore
    # write the total, amt correct and percentage score to md file
    markdown_contents += f"# Gatorgrade Insights\n\n**Project Name:** {Path.cwd().name}\n**Amount Correct:** {(json.get('amount_correct'))}/{num_checks} ({(json.get('percentage_score'))}%)\n"
    # split checks into passing and not passing
    for check in json.get("checks"):  # type: ignore
        # if the check is passing
        if check["status"]:
            passing_checks.append(check)
        # if the check is failing
        else:
            failing_checks.append(check)
    # give short info about passing checks
    markdown_contents += "\n## Passing Checks\n"
    for check in passing_checks:
        if "description" in check:
            markdown_contents += f"\n- [x] {check['description']}"
        else:
            markdown_contents += f"\n- [x] {check['check']}"
    # give extended information about failing checks
    markdown_contents += "\n\n## Failing Checks\n"
    # for each failing check, print out all related information
    for check in failing_checks:
        # for each key val pair in the check dictionary
        if "description" in check:
            markdown_contents += f"\n- [ ] {check['description']}"
        else:
            markdown_contents += f"\n- [ ] {check['check']}"
        if "options" in check:
            for i in check.get("options"):
                if "command" == i:
                    val = check["options"]["command"]
                    markdown_contents += f"\n\t- **command** {val}"
                if "fragment" == i:
                    val = check["options"]["fragment"]
                    markdown_contents += f"\n\t- **fragment:** {val}"
                if "tag" == i:
                    val = check["options"]["tag"]
                    markdown_contents += f"\n\t- **tag:** {val}"
                if "count" == i:
                    val = check["options"]["count"]
                    markdown_contents += f"\n\t- **count:** {val}"
                if "directory" == i:
                    val = check["options"]["directory"]
                    markdown_contents += f"\n\t- **directory:** {val}"
                if "file" == i:
                    val = check["options"]["file"]
                    markdown_contents += f"\n\t- **file:** {val}"
        elif "command" in check:
            val = check["command"]
            markdown_contents += f"\n\t- **command:** {val}"
        if "diagnostic" in check:
            markdown_contents += f"\n\t- **diagnostic:** {check['diagnostic']}"
        markdown_contents += "\n"
    return markdown_contents


def configure_report(
    report_params: Tuple[str, str, str], report_output_data_json: dict
):
    """Put together the contents of the report depending on the inputs of the user.

    Args:
        report_params: The details of what the user wants the report to look like
            report_params[0]: file or env
            report_params[1]: json or md
            report_params[2]: name of the file or env
        report_output_data: the json dictionary that will be used or converted to md
    """
    report_format = report_params[0]
    report_type = report_params[1]
    report_name = report_params[2]
    if report_type not in ("json", "md"):
        raise ValueError(
            "\n[red]The second argument of report has to be 'md' or 'json' "
        )
    # if the user wants markdown, get markdown content based on json
    if report_type == "md":
        report_output_data_md = create_markdown_report_file(report_output_data_json)
    # if the user wants the data stored in a file
    if report_format == "file":
        if report_type == "md":
            write_json_or_md_file(report_name, report_type, report_output_data_md)  # type: ignore
        else:
            write_json_or_md_file(report_name, report_type, report_output_data_json)
    # the user wants the data stored in an environment variable; do not attempt
    # to save to the environment variable if it does not exist in the environment
    elif report_format == "env":
        if report_name == "GITHUB_STEP_SUMMARY":
            env_file = os.getenv("GITHUB_STEP_SUMMARY", None)
            if env_file is not None:
                if report_type == "md":
                    write_json_or_md_file(env_file, report_type, report_output_data_md)  # type: ignore
                else:
                    write_json_or_md_file(
                        env_file, report_type, report_output_data_json
                    )
        # Add json report into the GITHUB_ENV environment variable for data collection purpose;
        # note that this is an undocumented side-effect of running gatorgrade with command-line
        # arguments that save data to the GITHUB_STEP_SUMMARY environment variable. The current
        # implementation of this approach should not cause the setting to fail when GatorGrade
        # is run with the same command-line for which it is normally run in a GitHub Actions
        # convert the data to a JSON string so that it can potentially be saved
        json_string = json.dumps(report_output_data_json)
        # check to see if the GITHUB_ENV environment variable is set
        env_file = os.getenv("GITHUB_ENV", None)
        # the environment variable is defined and thus it is acceptable
        # to write a key-value pair to the GITHUB_ENV environment file
        # (note that the comment on the previous line is correct; this
        # environment variable is a pointer to a file that allows for
        # key-value pairs in one step to be passed to the next step
        # inside of GitHub Actions and it is done through a file)
        if env_file is not None:
            # if it is, append the JSON string to the GITHUB_ENV file;
            # note that this step is specifically helpful when running
            # GatorGrade inside of a GitHub Actions workflow because
            # this variable called GITHUB_ENV is used to store environment
            # variables that are available to all of the subsequent steps
            with open(os.environ["GITHUB_ENV"], "a") as env_file:  # type: ignore
                env_file.write(f"JSON_REPORT={json_string}\n")  # type: ignore
    else:
        raise ValueError(
            "\n[red]The first argument of report has to be 'env' or 'file' "
        )


def write_json_or_md_file(file_name, content_type, content):
    """Write a markdown or json file."""
    # try to store content in a file with user chosen format
    try:
        # Second argument has to be json or md
        with open(file_name, "w", encoding="utf-8") as file:
            if content_type == "json":
                json.dump(content, file, indent=4)
            else:
                file.write(str(content))
        return True
    except Exception as e:
        raise ValueError(
            "\n[red]Can't open or write the target file, check if you provide a valid path"
        ) from e


def run_checks(
    checks: List[Union[ShellCheck, GatorGraderCheck]],
    report: Tuple[str, str, str],
    running_mode=False,
    no_status_bar=False,
) -> bool:
    """Run shell and GatorGrader checks and display whether each has passed or failed.

        Also, print a list of all failed checks with their diagnostics and a summary message that
        shows the overall fraction of passed checks.

    Args:
        checks: The list of shell and GatorGrader checks to run.
        running_mode: Convert the Progress Bar to update based on checks ran/not ran.
        no_status_bar: Option to completely disable all Progress Bar options.
    """
    results = []
    # run each of the checks
    # check how many tests are being ran
    total_checks = len(checks)
    # run checks with no progress bar
    if no_status_bar:
        for check in checks:
            result = None
            command_ran = None
            # run a shell check; this means
            # that it is going to run a command
            # in the shell as a part of a check;
            # store the command that ran in the
            # field called run_command that is
            # inside of a CheckResult object but
            # not initialized in the constructor
            if isinstance(check, ShellCheck):
                result = _run_shell_check(check)
                command_ran = check.command
                result.run_command = command_ran
            # run a check that GatorGrader implements
            elif isinstance(check, GatorGraderCheck):
                result = _run_gg_check(check)
                # check to see if there was a command in the
                # GatorGraderCheck. This code finds the index of the
                # word "--command" in the check.gg_args list if it
                # is available (it is not available for all of
                # the various types of GatorGraderCheck instances),
                # and then it adds 1 to that index to get the actual
                # command run and then stores that command in the
                # result.run_command field that is initialized to
                # an empty string in the constructor for CheckResult
                if "--command" in check.gg_args:
                    index_of_command = check.gg_args.index("--command")
                    index_of_new_command = int(index_of_command) + 1
                    result.run_command = check.gg_args[index_of_new_command]
            # there were results from running checks
            # and thus they must be displayed
            if result is not None:
                result.print()
                results.append(result)
    else:
        with Progress(
            TextColumn("[progress.description]{task.description}"),
            BarColumn(
                bar_width=40,
                style="red",
                complete_style="green",
                finished_style="green",
            ),
            TextColumn("[progress.percentage]{task.percentage:>3.0f}%"),
        ) as progress:
            # add a progress task for tracking
            task = progress.add_task("[green]Running checks", total=total_checks)
            # run each of the checks
            for check in checks:
                result = None
                command_ran = None
                if isinstance(check, ShellCheck):
                    result = _run_shell_check(check)
                    command_ran = check.command
                    result.run_command = command_ran
                # run a check that GatorGrader implements
                elif isinstance(check, GatorGraderCheck):
                    result = _run_gg_check(check)
                    # check to see if there was a command in the
                    # GatorGraderCheck. This code finds the index of the
                    # word "--command" in the check.gg_args list if it
                    # is available (it is not available for all of
                    # the various types of GatorGraderCheck instances),
                    # and then it adds 1 to that index to get the actual
                    # command run and then stores that command in the
                    # result.run_command field that is initialized to
                    # an empty string in the constructor for CheckResult
                    if "--command" in check.gg_args:
                        index_of_command = check.gg_args.index("--command")
                        index_of_new_command = int(index_of_command) + 1
                        result.run_command = check.gg_args[index_of_new_command]
                # there were results from running checks
                # and thus they must be displayed
                if result is not None:
                    result.print()
                    results.append(result)
                # update progress based on running_mode
                if running_mode:
                    progress.update(task, advance=1)
                else:
                    if result and result.passed:
                        progress.update(task, advance=1)
    # determine if there are failures and then display them
    failed_results = list(filter(lambda result: not result.passed, results))
    # print failures list if there are failures to print
    # and print what ShellCheck command that Gatorgrade ran
    if len(failed_results) > 0:
        print("\n-~-  FAILURES  -~-\n")
        for result in failed_results:
            # main.console.print("This is a result")
            # main.console.print(result)
            result.print(show_diagnostic=True)
            # this result is an instance of CheckResult
            # that has a run_command field that is some
            # value that is not the default of an empty
            # string and thus it should be displayed;
            # the idea is that displaying this run_command
            # will give the person using Gatorgrade a way
            # to quickly run the command that failed
            if result.run_command != "":
                rich.print(f"[blue]   → Run this command: [green]{result.run_command}")
    # determine how many of the checks passed and then
    # compute the total percentage of checks passed
    passed_count = len(results) - len(failed_results)
    # prevent division by zero if no results
    if len(results) == 0:
        percent = 0
    else:
        percent = round(passed_count / len(results) * 100)
    # if the report is wanted, create output in line with their specifications
    if all(report):
        report_output_data = create_report_json(passed_count, results, percent)
        configure_report(report, report_output_data)
    # compute summary results and display them in the console using the Panel
    # provided by Rich; this enables a border that resizes with the terminal;
    # note that there is one blank line between the prior output and the Panel
    summary = f"Passed {passed_count}/{len(results)} ({percent}%) of checks for {Path.cwd().name}!"
    summary_color = "green" if passed_count == len(results) else "bright_red"
    rich.print("")
    rich.print(Panel(summary, expand=False, title=None, style=summary_color))
    # determine whether or not the run was a success or not:
    # if all of the tests pass then the function returns True;
    # otherwise the function must return False
    summary_status = True if passed_count == len(results) else False
    return summary_status



"""Generates a dictionary of shell and gator grader command options from a list of dict checks."""

import os
from typing import List
from typing import Union

from .checks import GatorGraderCheck
from .checks import ShellCheck
from .in_file_path import CheckData


# pylint: disable=too-many-nested-blocks
def generate_checks(
    check_data_list: List[CheckData],
) -> List[Union[ShellCheck, GatorGraderCheck]]:
    """Generate a list of checks based on check data from the configuration file.

    Args:
        check_data_list: A list of CheckData that each represent a check from the
            configuration file.

    Returns:
        A list of ShellChecks and GatorGraderChecks.
    """
    checks: List[Union[ShellCheck, GatorGraderCheck]] = []
    for check_data in check_data_list:
        # If the check has a `command` key, then it is a shell check
        if "command" in check_data.check:
            checks.append(
                ShellCheck(
                    command=check_data.check.get("command"),
                    description=check_data.check.get("description"),
                    json_info=check_data.check,
                )
            )
        # Otherwise, it is a GatorGrader check
        else:
            gg_args = []
            # Add description option if in data
            description = check_data.check.get("description")
            if description is not None:
                gg_args.extend(["--description", str(description)])
            # Always add name of check, which should be in data
            gg_args.append(str(check_data.check.get("check")))
            # Add any additional options
            options = check_data.check.get("options")
            if options is not None:
                for option in options:
                    # If option should be a flag (i.e. its value is the `True` boolean),
                    # add only the option without a value
                    option_value = options[option]
                    if isinstance(option_value, bool):
                        if option_value:
                            gg_args.append(f"--{option}")
                    # Otherwise, add both the option and its value
                    else:
                        gg_args.extend([f"--{option}", str(option_value)])
            # Add directory and file if file context in data
            if check_data.file_context is not None:
                # Get the file and directory using os
                dirname, filename = os.path.split(check_data.file_context)
                if dirname == "":
                    dirname = "."
                gg_args.extend(["--directory", dirname, "--file", filename])
            checks.append(GatorGraderCheck(gg_args=gg_args, json_info=check_data.check))

    return checks



"""Define check classes."""

from typing import List


class ShellCheck:  # pylint: disable=too-few-public-methods
    """Represent a shell check."""

    def __init__(self, command: str, description: str = None, json_info=None):  # type: ignore
        """Construct a ShellCheck.

        Args:
            command: The command to run in a shell.
            description: The description to use in output.
                If no description is given, the command is used as the description.
            json_info: The all-encompassing check information to include in json output.
                If none is given, command is used
        """
        self.command = command
        self.description = description if description is not None else command
        self.json_info = json_info


class GatorGraderCheck:  # pylint: disable=too-few-public-methods
    """Represent a GatorGrader check."""

    def __init__(self, gg_args: List[str], json_info):
        """Construct a GatorGraderCheck.

        Args:
            gg_args: The list of arguments to pass to GatorGrader.
            json_info: The all-encompassing check information to include in json output.
        """
        self.gg_args = gg_args
        self.json_info = json_info



"""Define check result class."""

from typing import Union
import rich


class CheckResult:  # pylint: disable=too-few-public-methods
    """Represent the result of running a check."""

    def __init__(
        self,
        passed: bool,
        description: str,
        json_info,
        path: Union[str, None] = None,
        diagnostic: str = "No diagnostic message available",
    ):
        """Construct a CheckResult.

        Args:
            passed: The passed or failed status of the check result. If true, indicates that the
                check has passed.
            description: The description to use in output.
            json_info: the overall information to be included in json output
            diagnostic: The message to use in output if the check has failed.
        """
        self.passed = passed
        self.description = description
        self.json_info = json_info
        self.diagnostic = diagnostic
        self.path = path
        self.run_command = ""

    def display_result(self, show_diagnostic: bool = False) -> str:
        """Print check's passed or failed status, description, and, optionally, diagnostic message.

        If no diagnostic message is available, then the output will say so.

        Args:
            show_diagnostic: If true, show the diagnostic message if the check has failed.
                Defaults to false.
        """
        icon = "✓" if self.passed else "✕"
        icon_color = "green" if self.passed else "red"
        message = f"[{icon_color}]{icon}[/]  {self.description}"
        if not self.passed and show_diagnostic:
            message += f"\n[yellow]   → {self.diagnostic}"
        return message

    def __repr__(self):
        return f"CheckResult(passed={self.passed}, description='{self.description}', json_info={self.json_info}, path='{self.path}', diagnostic='{self.diagnostic}', run_command='{self.run_command}')"

    def __str__(self, show_diagnostic: bool = False) -> str:
        """Print check's passed or failed status, description, and, optionally, diagnostic message.

        If no diagnostic message is available, then the output will say so.

        Args:
            show_diagnostic: If true, show the diagnostic message if the check has failed.
                Defaults to false.
        """
        message = self.display_result(show_diagnostic)
        return message

    def print(self, show_diagnostic: bool = False) -> None:
        """Print check's passed or failed status, description, and, optionally, diagnostic message.

        If no diagnostic message is available, then the output will say so.

        Args:
            show_diagnostic: If true, show the diagnostic message if the check has failed.
                Defaults to false.
        """
        message = self.display_result(show_diagnostic)
        rich.print(message)



"""Returns the list of commands to be run through gatorgrader."""

from pathlib import Path

from gatorgrade.input.command_line_generator import generate_checks
from gatorgrade.input.in_file_path import parse_yaml_file
from gatorgrade.input.in_file_path import reformat_yaml_data


def parse_config(file: Path):
    """Parse the input yaml file and generate specified checks.

    Args:
        file: Yaml file containing gatorgrade and shell command checks
    Returns:
        Returns a dictionary that specifies shell commands and gatorgrade commands
    """
    # parse the YAML file using parse_yaml_file provided by gatorgrade
    parsed_yaml_file = parse_yaml_file(file)
    # the parsed YAML file contains some contents in a list and thus
    # the tool should generate a GatorGrader check for each element in list
    if len(parsed_yaml_file) > 0:
        # after reformatting the parse YAML file,
        # use it to generate all of the checks;
        # these will be valid checks that are now
        # ready for execution with this tool
        parse_con = generate_checks(reformat_yaml_data(parsed_yaml_file))
        return parse_con
    # return an empty list because of the fact that the
    # parsing process did not return a list with content;
    # allow the calling function to handle the empty list
    return []
